@@ -166,11 +166,16 @@ tests =
   -- tall layout
 
   ,("tile 1 window fullsize", property prop_tile_fullscreen)
+  ,("tile max ratio",         property prop_tile_max_ratio)
+  ,("tile min ratio",         property prop_tile_min_ratio)
   ,("tiles never overlap",    property prop_tile_non_overlap)
   ,("split horizontal",       property prop_split_horizontal)
   ,("split vertical",         property prop_split_vertical)
 
-  ,("pure layout tall",       property prop_purelayout_tall)
+  ,("pure layout tall",            property prop_purelayout_tall)
+    {- Following two test cases should be automatically generated by QuickCheck ideally, but it fails. -}
+  ,("pure layout tall: ratio = 0", property (\n d rect -> prop_purelayout_tall n d 0 rect))
+  ,("pure layout tall: ratio = 1", property (\n d rect -> prop_purelayout_tall n d 1 rect))
   ,("send shrink    tall",    property prop_shrink_tall)
   ,("send expand    tall",    property prop_expand_tall)
   ,("send incmaster tall",    property prop_incmaster_tall)
